/* CS 472/672 Assignment #3 - ilu.h file */

#ifndef ILU_H
#define ILU_H

/*                                                     */
/* Sparse row format data structure for the ILU matrix */
/*                                                     */
/*   
   Each row of the ILU matrix is held as a sparse_row structure
   for k=0, ..., nz-1
      jaf[k] = col index
      af[k]  = value
      k = diag is the location of the diagonal of the row  
          i.e. if k = row.diag => af[k] = diag entry 
*/
struct sparse_row {
   double *af;        /* Coefficient, as with a */
   int    *jaf;       /* Column index, as with ja */
   int    nz;         /* Number of nonzeros in the row */
   int    diag;       /* Where the diagonal coefficient is stored */
};


/*                             */
/* Incomplete LU factorization */
/*                             */

void ilufac(int N, int *ia, int *ja, double *a, int *order, int *invord, struct sparse_row *ILU);
/*
   input:

      N       number of unknowns
      ia, ja  stucture of a
      a       real values of a
      order   order[new_order] = old_order
      invord  invord[old_order] = new_order
      ILU     contains info about symbolic factors of the ILU

   output:

      ILU     now contains real ILU as well as symbolic
*/


/*             */
/* Solve phase */
/*             */

void ilusol(int N, double *x, double *b, int *order, struct sparse_row *ILU);
/*
   input:

      N      number of unknowns
      b      rhs
      order  order[new_order] = old_order
      ILU    sparse_row structure containing ILU

   output:

      x      solution in the original order
*/


/*                                   */
/* Symbolic incomplete factorization */
/*                                   */

void ilusym(int N, int *ia, int *ja, int *order, int *invord, struct sparse_row *ILU);
/*
   input:

     N       number of unknowns
     ia, ja  structure of a
     order   order[new_order] = old_order
     invord  invord[old_order] = new_order

  output:

     ILU     sparse_row stucture contains symbolic ILU info
*/


/* Auxilliary functions */
void shell(int *v, int n);
void sortq(int *v, int left, int right);
void swap(int *v, int i, int j);
void clean(int N, struct sparse_row *ILU);

#endif

