/* CS472/672 Assignment #3 - itsolver.c file */

#include <math.h>
#include <stdio.h>
#include <malloc.h>
#include "itsolver.h"
#include "ilu.h"


/*                    */
/*  Jacobi iteration  */
/*                    */
void Jacobi(int N,int *ia,int *ja,double *a,double *b,double *x,int maxiter, double tol,int *iter,double *res)
/*
   input:

      N        number of unknowns
      ia, ja   structure of a
      a        real values of a
      b        right-hand side
      x        initial guess
      maxiter  maximum number of iterations
      tol      convergence tolerance

    output:

      x        final solution
      iter     final number of iterations
      res      final residual norm
*/
{
   int    i, j, k;
   double *x_old, diag, r_norm, b_norm;


   /* Compute the norms of initial residual and rhs. */
   b_norm = 0.0;
   for (i=0; i<N; i++) b_norm += b[i]*b[i];
   b_norm = sqrt(b_norm);
   r_norm = ResNorm(N, ia, ja, a, b, x);

   x_old = (double *) malloc(N*sizeof(double));

   k=0;

   /* Jacobi iteration */
   while ((r_norm > tol*b_norm) && (k < maxiter)) {

      for (i=0; i<N; i++) x_old[i] = x[i];

      for (i=0; i<N; i++) {
         x[i] = b[i];
         for (j=ia[i]; j<ia[i+1]; j++) {
            if (ja[j] != i) {
               x[i] -= x_old[ja[j]] * a[j];
            }
            else {
               diag = a[j];
            }
         }
         x[i] /= diag;
      }

      k++;

      /* Compute the residual norm only every 100 iterations. */
      if (k%100==0) r_norm = ResNorm(N, ia, ja, a, b, x);
   }

   *iter = k;
   *res = r_norm;

   free(x_old);

}


/*                          */
/*  Gauss-Seidel iteration  */
/*                          */
void GS(int N,int *ia,int *ja,double *a,double *b,double *x,int maxiter,double tol,int *iter,double *res)
{

   printf("You have to write the function GS.\n");

}


/*                 */
/*  SOR iteration  */
/*                 */
void SOR(int N,int *ia,int *ja,double *a,double *b,double *x,double omega,int maxiter,double tol,int *iter,double *res)
{

   printf("You have to write the function SOR.\n");

}


/*                */
/*  CG iteration  */
/*                */
void CG(int N,int *ia,int *ja,double *a,double *b,double *x,int maxiter,double tol,int *iter,double *res)
{

   printf("You have to write the function CG.\n");

}


/*                */
/*  PCG iteration  */
/*                */
void PCG(int N,int *ia,int *ja,double *a,double *b,double *x,int *order,struct sparse_row *ILU,int maxiter,double tol,int *iter, double *r_norm)
{

   printf("You have to write the function PCG.\n");

}


/* Compute the residual norm: ||b-Ax|| */
double ResNorm(int N, int *ia, int *ja, double *a, double *b, double *x)
{
   int    i, j;
   double ri, res;

   res = 0.0;
   for (i=0; i<N; i++) {
      ri = b[i];
      for (j=ia[i]; j<ia[i+1]; j++) {
         ri -= x[ja[j]] * a[j];
      }
      res += ri*ri;
   }
   res = sqrt(res);

   return res;
}


/* Compute the residual vector: r=b-Ax */
void Residual(int N,int *ia,int *ja,double *a,double *b,double *x,double *r)
{
   printf("You might want to write the function Residual.\n");
}


/* Dot product of x and y */
void DotProduct(int N,double *x,double *y,double *xy)
{
   printf("You might want to write the function DotProduct.\n");
}


/* Compute z = alpha x + y */
void DAXPY(int N,double alpha,double *x,double *y,double *z)
{
   printf("You might want to write the function DAXPY.\n");
}


/* Compute matrix-vector product: y = Ax */
void Matvec(int N,int *ia,int *ja,double *a,double *x,double *y)
{
   printf("You might want to write the function Matvec.\n");
}
