/* CS 472/672 Assignment #3 - itsolver.h file */

#include "ilu.h"

#ifndef ITSOLVER_H
#define ITSOLVER_H

/*
   Parameters for iterative solvers.

   input:

      N        number of unknowns
      ia, ja   structure of a
      a        real values of a
      b        right-hand side
      x        initial guess
      maxiter  maximum number of iterations
      tol      convergence tolerance
      omega    over-relaxation parameter (SOR only)
      order    order vector (PCG only)
      ILU      incomplete LU matrix (PCG only)

    output:

      x        final solution
      iter     final number of iterations
      res      final residual norm
*/

/* Jacobi */
void Jacobi(int N,int *ia,int *ja,double *a,double *b,double *x,int maxiter,double tol,int *iter,double *res);

/* Gauss-Seidel */
void GS(int N,int *ia,int *ja,double *a,double *b,double *x,int maxiter,double tol,int *iter,double *res);

/* SOR */
void SOR(int N,int *ia,int *ja,double *a,double *b,double *x,double omega,int maxiter,double tol,int *iter,double *res);

/* Conjugate gradient */
void CG(int N,int *ia,int *ja,double *a,double *b,double *x,int maxiter,double tol,int *iter,double *res);

/* Preconditioned conjugate gradient */
void PCG(int N,int *ia,int *ja,double *a,double *b,double *x,int *order,struct sparse_row *ILU,int maxiter,double tol,int *iter,double *res);

/* Compute the residual norm: ||b-Ax|| */
double ResNorm(int N,int *ia,int *ja,double *a,double *b,double *x);


/* You may find it useful to write the following functions. */

/* Compute the residual: r=b-Ax */
void Residual(int N,int *ia,int *ja,double *a,double *b,double *x,double *r);

/* Dot producto of x and y */
void DotProduct(int N,double *x,double *y,double *xy);

/* Compute z = alpha x + y */
void DAXPY(int N,double alpha,double *x,double *y,double *z);

/* Compute matrix-vector multiply: y = Ax */
void Matvec(int N,int *ia,int *ja,double *a,double *x,double *y);

#endif

